/*	reduce.c

	AFAPI associative reduction function definitions for
	PAPERS_JR.  Because NPROC=2, each of the reductions is able to
	be done using a single putGet.

	July 1996 by H. Dietz
*/


#include "afapi.h"
#include "inline.h"

/*	p_reduceAnd() functions...
*/

p_1u
p_reduceAnd1u(register p_1u datum)
{
	return(p__all(datum));
}


#define	P_DECLARE(suffix) \
p_##suffix \
p_reduceAnd##suffix(register p_##suffix datum) \
{ \
	return(datum & p_putGet##suffix(datum, (IPROC ^ 1))); \
}

P_DECLARE(8)
P_DECLARE(8u)
P_DECLARE(16)
P_DECLARE(16u)
P_DECLARE(32)
P_DECLARE(32u)
P_DECLARE(64)
P_DECLARE(64u)

#undef	P_DECLARE


p_bar
p_reduceAndbar(register p_bar datum)
{
	/* Barrier fits within a nybble... */
	return((p_bar) datum & p__swap(datum));
}


p_ptr
p_reduceAndptr(register p_ptr datum)
{
	return((p_ptr) (((p_32u) datum) &
			p_putGet32u(((p_32u) datum), (IPROC ^ 1))));
}

/*	p_reduceOr() functions...
*/

inline p_1u
p_reduceOr1u(register p_1u datum)
{
	return(p__any(datum));
}


#define	P_DECLARE(suffix) \
inline p_##suffix \
p_reduceOr##suffix(register p_##suffix datum) \
{ \
	return(datum | p_putGet##suffix(datum, (IPROC ^ 1))); \
}

P_DECLARE(8)
P_DECLARE(8u)
P_DECLARE(16)
P_DECLARE(16u)
P_DECLARE(32)
P_DECLARE(32u)
P_DECLARE(64)
P_DECLARE(64u)

#undef	P_DECLARE


p_bar
p_reduceOrbar(register p_bar datum)
{
	/* Barrier fits within a nybble... */
	return((p_bar) datum | p__swap(datum));
}


p_ptr
p_reduceOrptr(register p_ptr datum)
{
	return((p_ptr) (((p_32u) datum) |
			p_putGet32u(((p_32u) datum), (IPROC ^ 1))));
}

/*	p_reduceXor() functions...
*/

p_1u
p_reduceXor1u(register p_1u datum)
{
	return(p_count(datum) & 1);
}


#define	P_DECLARE(suffix) \
p_##suffix \
p_reduceXor##suffix(register p_##suffix datum) \
{ \
	return(datum ^ p_putGet##suffix(datum, (IPROC ^ 1))); \
}

P_DECLARE(8)
P_DECLARE(8u)
P_DECLARE(16)
P_DECLARE(16u)
P_DECLARE(32)
P_DECLARE(32u)
P_DECLARE(64)
P_DECLARE(64u)

#undef	P_DECLARE


p_bar
p_reduceXorbar(register p_bar datum)
{
	/* Barrier fits within a nybble... */
	return((p_bar) datum ^ p__swap(datum));
}


p_ptr
p_reduceXorptr(register p_ptr datum)
{
	return((p_ptr) (((p_32u) datum) ^
			p_putGet32u(((p_32u) datum), (IPROC ^ 1))));
}

/*	p_reduceMin() functions...
*/

#define	P_DECLARE(suffix) \
inline p_##suffix \
p_reduceMin##suffix(register p_##suffix datum) \
{ \
	register p_##suffix t = p_putGet##suffix(datum, (IPROC ^ 1)); \
	return((datum < t) ? datum : t); \
}

P_ALPHA

#undef	P_DECLARE

/*	p_reduceMax() functions...
*/

#define	P_DECLARE(suffix) \
inline p_##suffix \
p_reduceMax##suffix(register p_##suffix datum) \
{ \
	register p_##suffix t = p_putGet##suffix(datum, (IPROC ^ 1)); \
	return((datum > t) ? datum : t); \
}

P_ALPHA

#undef	P_DECLARE

/*	p_reduceAdd() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_reduceAdd##suffix(register p_##suffix datum) \
{ \
	return(datum + p_putGet##suffix(datum, (IPROC ^ 1))); \
}

P_GAMMA

#undef	P_DECLARE

/*	p_reduceMul() functions...
*/

#define	P_DECLARE(suffix) \
p_##suffix \
p_reduceMul##suffix(register p_##suffix datum) \
{ \
	return(datum * p_putGet##suffix(datum, (IPROC ^ 1))); \
}

P_GAMMA

#undef	P_DECLARE
