/*	vote.c

	AFAPI voting and scheduling function definitions for
	PAPERS_JR.  Not much to voting when there are only 2 PEs.

	July 1996 by H. Dietz
*/


#include "afapi.h"
#include "inline.h"

/*	p_first()...

	Return the IPROC of the first lowest-numbered PE with a yes
	vote.  It is easy to do, but a tad magical....
*/

int
p_first(register p_1u f)
{
	/* Not real hard, given that NPROC=2  ;-) */
	int pat[2];

	f = (f != 0);
	pat[IPROC] = f;
	pat[IPROC ^ 1] = p__swap(f);

	if (pat[0]) return(0);
	if (pat[1]) return(1);
	return(2);
}

/*	p_count()...

	Count how many voted yes.  Also some magic here....
*/

inline int
p_count(register p_1u f)
{
	/* Not real hard, given that NPROC=2  ;-) */

	f = (f != 0);
	return(f + p__swap(f));
}

/*	p_quantify()...

	Quantify the number of PEs voting yes; the return is 0, 1, 2
	(really >1), or NPROC.  The value is allowed to distinguish
	more than these counts, and actually returns a precise count
	in some implementations.  In fact, for NPROC < 4, the precise
	count and quantify are by definition equivalent.
*/

int
p_quantify(register p_1u f)
{
	return(p_count(f));
}

/*	p_vote()...

	We just need to see how the other guy voted.  Notice that
	NPROC fits in just 2 bits, so a single p__swap() works.
*/

p_bar
p_vote(register int pe)
{
	return(((pe == IPROC) << IPROC) |
	       ((p__swap(pe) == IPROC) << (IPROC ^ 1)));
}

/*	p_voteCount()...

	We just need to see how the other guy voted.  Notice that
	NPROC fits in just 2 bits, so a single p__swap() works.
*/

int
p_voteCount(register int pe)
{
	return((pe == IPROC) +
	       (p__swap(pe) == IPROC));
}

/*	p_match()...
*/

#define	P_DECLARE(suffix) \
inline p_bar \
p_match##suffix(register p_##suffix datum) \
{ \
	/* Note: we always match ourself */ \
	return((1 << IPROC) | \
	       ((p_putGet##suffix(datum, (IPROC ^ 1)) == datum) << \
	        (IPROC ^ 1))); \
}

P_BETA

#undef	P_DECLARE

/*	p_matchCount()...
*/

#define	P_DECLARE(suffix) \
int \
p_matchCount##suffix(register p_##suffix datum) \
{ \
	/* Note: we always match ourself */ \
	return(1 + \
	       (p_putGet##suffix(datum, (IPROC ^ 1)) == datum)); \
}

P_BETA

#undef	P_DECLARE

/*	p_rank()...

	Use putGet to get his value.  For NPROC=2, easy....
*/

#define	P_DECLARE(suffix) \
int \
p_rank##suffix(register p_##suffix datum) \
{ \
	register p_##suffix t = p_putGet##suffix(datum, (IPROC ^ 1)); \
	if (datum < t) return(0); \
	if (datum == t) return(IPROC); \
	return(1); \
}

P_ALPHA

#undef	P_DECLARE
